@@ -224,7 +224,7 @@ static int fio_spliceio_queue(struct thread_data *td, struct io_u *io_u)
 	} else if (io_u->ddir == DDIR_WRITE)
 		ret = fio_splice_write(td, io_u);
 	else
-		ret = fsync(io_u->file->fd);
+		ret = do_io_u_sync(td, io_u);
 
 	if (ret != (int) io_u->xfer_buflen) {
 		if (ret >= 0) {
@@ -297,7 +297,7 @@ static struct ioengine_ops ioengine = {
 	.open_file	= generic_open_file,
 	.close_file	= generic_close_file,
 	.get_file_size	= generic_get_file_size,
-	.flags		= FIO_SYNCIO,
+	.flags		= FIO_SYNCIO | FIO_PIPEIO,
 };
 
 #else /* FIO_HAVE_SPLICE */
