@@ -15,6 +15,7 @@ enum {
 	VERIFY_SHA256,			/* sha256 sum data blocks */
 	VERIFY_SHA512,			/* sha512 sum data blocks */
 	VERIFY_META,			/* block_num, timestamp etc. */
+	VERIFY_SHA1,			/* sha1 sum data blocks */
 	VERIFY_NULL,			/* pretend to verify */
 };
 
@@ -30,13 +31,16 @@ struct verify_header {
 };
 
 struct vhdr_md5 {
-	uint32_t md5_digest[16];
+	uint32_t md5_digest[4];
 };
 struct vhdr_sha512 {
 	uint8_t sha512[128];
 };
 struct vhdr_sha256 {
-	uint8_t sha256[128];
+	uint8_t sha256[64];
+};
+struct vhdr_sha1 {
+	uint32_t sha1[5];
 };
 struct vhdr_crc64 {
 	uint64_t crc64;
